import sys
import os
import dropbox
from dropbox.exceptions import ApiError
from prometheus_client import start_http_server, Gauge
import time

# How much time in seconds we are going to sleep
sleep_time = 5

def process_folder_entries(current_state, entries):
    n = 0
    for entry in entries:
        if type(entry) == dropbox.files.FileMetadata:
            n += 1
        elif type(entry) == dropbox.files.FolderMetadata:
            result = dbx.files_list_folder(path=entry.path_display)
            t = process_folder_entries({}, result.entries)
            n += t

        if isinstance(entry, dropbox.files.FileMetadata):
            current_state[entry.path_lower] = entry
        elif isinstance(entry, dropbox.files.DeletedMetadata):
            current_state.pop(entry.path_lower, None)
    return n

def process_folders(current_state, entries):
    n = 0
    for entry in entries:
        if type(entry) == dropbox.files.FolderMetadata:
            n += 1
            result = dbx.files_list_folder(path=entry.path_display)
            t = process_folders({}, result.entries)
            n += t

        if isinstance(entry, dropbox.files.FileMetadata):
            current_state[entry.path_lower] = entry
        elif isinstance(entry, dropbox.files.DeletedMetadata):
            current_state.pop(entry.path_lower, None)
    return n

def path_exists(path):
    try:
        dbx.files_get_metadata(path)
        return True
    except ApiError as e:
        if e.error.get_path().is_not_found():
            return False
        raise

access_token = os.environ.get('DROPBOX_TOKEN')
if access_token == None:
    print("DROPBOX_TOKEN is not set!")
    sys.exit()
dbx = dropbox.Dropbox(access_token)

# The port number for Prometheus to connect and get the metrics
start_http_server(2345)

nFiles = Gauge('number_of_files', 'Number of Files')
nFolders = Gauge('number_of_folders', 'Number of Folders')

# Create a loop to keep reading number of files
while True:
    result = dbx.files_list_folder(path="/LXF")
    nfiles_current = process_folder_entries({}, result.entries)
    print("Found", nfiles_current, "files.", end = '\n')

    nfolders_current = process_folders({}, result.entries)
    print("Found", nfolders_current, "folders.", end = '\n')

    nFiles.set(nfiles_current)
    nFolders.set(nfolders_current)
    # Sleep for a while
    time.sleep(sleep_time)
